import { Category2 } from 'fp-ts/lib/Category'
import { Invariant2 } from 'fp-ts/lib/Invariant'
import { Lens } from './Lens'
import { Optional } from './Optional'
import { Prism } from './Prism'
import { Traversal } from './Traversal'
/**
 * @category model
 * @since 2.3.0
 */
export interface Iso<S, A> {
  readonly get: (s: S) => A
  readonly reverseGet: (a: A) => S
}
/**
 * @category constructors
 * @since 2.3.0
 */
export declare const id: <S>() => Iso<S, S>
/**
 * View an `Iso` as a `Lens`
 *
 * @category converters
 * @since 2.3.0
 */
export declare const asLens: <S, A>(sa: Iso<S, A>) => Lens<S, A>
/**
 * View an `Iso` as a `Prism`
 *
 * @category converters
 * @since 2.3.0
 */
export declare const asPrism: <S, A>(sa: Iso<S, A>) => Prism<S, A>
/**
 * View an `Iso` as a `Optional`
 *
 * @category converters
 * @since 2.3.0
 */
export declare const asOptional: <S, A>(sa: Iso<S, A>) => Optional<S, A>
/**
 * View an `Iso` as a `Traversal`
 *
 * @category converters
 * @since 2.3.0
 */
export declare const asTraversal: <S, A>(sa: Iso<S, A>) => Traversal<S, A>
/**
 * Compose an `Iso` with an `Iso`
 *
 * @category compositions
 * @since 2.3.0
 */
export declare const compose: <A, B>(ab: Iso<A, B>) => <S>(sa: Iso<S, A>) => Iso<S, B>
/**
 * @category constructors
 * @since 2.3.0
 */
export declare const reverse: <S, A>(sa: Iso<S, A>) => Iso<A, S>
/**
 * @category combinators
 * @since 2.3.0
 */
export declare const modify: <A>(f: (a: A) => A) => <S>(sa: Iso<S, A>) => (s: S) => S
/**
 * @category Invariant
 * @since 2.3.0
 */
export declare const imap: <A, B>(f: (a: A) => B, g: (b: B) => A) => <S>(sa: Iso<S, A>) => Iso<S, B>
/**
 * @category instances
 * @since 2.3.0
 */
export declare const URI = 'monocle-ts/Iso'
/**
 * @category instances
 * @since 2.3.0
 */
export declare type URI = typeof URI
declare module 'fp-ts/lib/HKT' {
  interface URItoKind2<E, A> {
    readonly [URI]: Iso<E, A>
  }
}
/**
 * @category instances
 * @since 2.3.0
 */
export declare const invariantIso: Invariant2<URI>
/**
 * @category instances
 * @since 2.3.0
 */
export declare const categoryIso: Category2<URI>
